// Debugging alerts thru # 1

// UTILITIES DEFINED ELSEWHERE
// firstWord(s) top.js

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function jmolAtomCount(atomexp)
{
	var toJmol = "{(" + atomexp + ")";

	if (modelCount > 1)
		toJmol += " and model=1";

	toJmol += "}.count";

	return jmolEvaluate(toJmol);
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function match(name, subname)
{
	if (name.toLowerCase().indexOf(subname.toLowerCase()) == -1)
		return false;
	return true;
}
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
function setCharAt(str,index,chr)
{
    if(index > str.length-1) return str;
    return str.substr(0,index) + chr + str.substr(index+1);
}
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
function cutFromTo(str, start, endbefore)
{
	var ist = str.indexOf(start);
	if (ist == -1)
		return "";
	var ieb = str.indexOf(endbefore, ist);
	return str.substring(ist, ieb);
}
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
function capitalize(str)
{
	return str.charAt(0).toUpperCase() + str.substring(1).toLowerCase();
}
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*
function countChar(str, chr)
{
	var ctc = 0;
	ichr = 0;
	while ( (ichr = str.indexOf(chr, ichr)) != -1)
	{
		ichr++;
		ctc++;		
	}
	return (ctc);
}
*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
// TEST CASES
//  PROTEIN
//   1d66 protein, dna
//   3zen protein, unk, missing (with long chain >3,000!)
//   4fzv A: MSE, B: UNK, missing in both including missing UNK
//   4dgw C: MSE and UNK and missing
//   3pq1 MSE and UNK and VARIABLE missing in seqID chains.
//   3ou0 has 2 peptide chains of 100% UNK!
//   1w5u has 8 nonstd residues/chain (gramicidin).

//   4ev2 has nonstd TYQ

//  NUCLEIC
//   4b5r rna
//   1evv rna with 14 nonstd, no missing.
//   1i2x, 104d, 2l7d dna/rna hybrid

var stdAA = ["ALA", "ARG", "ASN", "ASP", "CYS", "GLN", "GLU", 
 "GLY", "HIS", "ILE", "LEU", "LYS", "MET", "PHE", "PRO", "SER",
	"THR", "TRP", "TYR", "VAL", "ASX", "GLX"];

var stdDNA = ["DA", "DC", "DG", "DT", "DU", "DI"];

var stdRNA = ["A", "C", "G", "T", "U", "I"];

var MSE = new Array;
MSE[0] = "MSE";

var UNK = new Array;
UNK[0] = "UNK";

var mseLink = '<a href="http://en.wikipedia.org/wiki/Selenomethionine" ' +
'target="_blank">';

var cseLink = '<a href="http://proteopedia.org/w/Selenocysteine" ' +
'target="_blank">';

var nsrlinked = '<a href="javascript: showLigNSRHelp()">non-standard \
residue(s)</a>';

var nonStandardResidueCount = 0;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
function getChainType(res)
{
	var clen = res.length;
	
	var p = countMatchingWords(res, stdAA);

	if (p == clen)
		return ("Protein");

	var d = countMatchingWords(res, stdDNA);
	if (d == clen)
		return ("DNA");

	var r = countMatchingWords(res, stdRNA);
	if (r == clen)
		return ("RNA");

	// THERE ARE NONSTANDARD RESIDUES OR UNK

	var unk = countMatchingWords(res, UNK);
	var mse = countMatchingWords(res, MSE);
	var notmse = clen - p - mse - unk;

	nonStandardResidueCount += (notmse + mse);

//	alert("util.js #1 clen=" + clen + " p=" + p + " mse=" + mse +
//		" notmse=" + notmse + " unk=" + unk);

	if (p > 0) // in Mar 2013, no entries contain 1 chain with protein and
		// DNA or RNA
	{
		var pmsg = "Protein";

		if (mse > 0)
		{
			pmsg += " including " + mse + " " + mseLink +
				"selenomethionines</a> (MSE)";

			if (notmse > 0)
				pmsg += " and " + notmse + " other " + nsrlinked;
		}
		else if (notmse > 0)
			pmsg += " including " + notmse + " " + nsrlinked + " (no " +
				mseLink + "selenomethonine</a> [MSE])";

		if (unk > 0)
		{
			if (mse > 0 || notmse > 0)
				pmsg += " and ";
			else
				pmsg += " including ";
			pmsg += unk + " unknown residue(s) [UNK]";
		}

		return pmsg;
	}

	// NUCLEIC WITH NONSTD OR UNK
	// IF NONSTD = 0 WE WILL SAY "with 0 non-standard residue(s)" BECAUSE
	// THEN THERE ARE UNKs.
	nonstd = clen - (d + r + unk);
	var unkmsg = "";
	if (unk > 0)
		unkmsg = " and " + unk + " Unknown residue(s) [UNK]";

	if (d > 0 && r > 0)
	{
		var hmsg = "DNA (" + d + ") and RNA (" + r + ")";
		if (nonstd > 0)
			hmsg += " with " + nonstd + " " + nsrlinked;

		return hmsg + unkmsg;
	}

	if (d > 0 && r == 0)
		return ("DNA with " + nonstd + " " + nsrlinked + unkmsg);

	if (r > 0 && d == 0)
		return ("RNA with " + nonstd + " " + nsrlinked + unkmsg);

	if (unk == clen)
		return ("100% unknown residue(s) [UNK]");
	else
		return ("Uncertain (not protein, DNA nor RNA)" +
			unkmsg.replace(/ and /, " with "));
}
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
// both words and tomatch are arrays
function countMatchingWords(words, tomatch)
{
	var mw = 0;
	for (var iw = 0; iw < words.length; iw++)
	{
		for (var im = 0; im < tomatch.length; im++)
		{
			if (words[iw] == tomatch[im])
			{
				mw++;
				break;
			}
		}
	}

	return (mw);
}
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
function singleSpace(str)
{
	while (str.indexOf("  ") != -1)
	{
		str = str.replace(/  /g, " ");
	}

	return (str);
}
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
// 3 FUNX FROM PE
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

function clean_word_string(to_do)
{
	// replace newlines with spaces
	var re = /\n/g;
	to_do = to_do.replace(re, " ");
	var re = /\r/g;
	to_do = to_do.replace(re, " ");

	// replace tabs with spaces
	var re = /\t/g;
	to_do = to_do.replace(re, " ");

	re = /  /g; // two spaces
	// collapse any multiple spaces to single spaces
	while (to_do.indexOf("  ") != -1)
		to_do = to_do.replace(re, " ");

	// trim off leading spaces
	while (to_do.charAt(0) == " ")
		to_do = to_do.substring(1);
	return to_do;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function clean_and_trim_ends(s)
{
	// \t,\n to spaces, spaces singularized, leading spaces trimmed
	s = clean_word_string(s);
	s = trim_ends(s);
	return s;	
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function trim_ends(s)
{
	var c1

	// trim leading end
	while (true)
	{
		if (s.length == 0)
			break;
		c1 = s.charAt(0);
		if (c1 == " " || c1 == "\n" || c1 == "\t" || c1 == "\r")		
			s = s.substring(1);
		else
			break;
	}

	// trim trailing end
	while (true)
	{
		if (s.length == 0)
			break;
		c1 = s.charAt(s.length - 1);
		if (c1 == " " || c1 == "\n" || c1 == "\t" || c1 == "\r")		
			s = s.substring(0, s.length - 1);
		else
			break;
	}

	return s;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function isValidPDBId(pid)
{
	if (pid.length != 4)
		return(false);

	pid = pid.toUpperCase();

	// First character must be 1-9
	if (pid.charAt(0) < '1' || pid.charAt(0) > '9')
		return(false);
	// Last three characters must be 1-9 or A-Z
	if (!pdbIdAlphaNum(pid, 1))
		return(false);
	if (!pdbIdAlphaNum(pid, 2))
		return(false);
	if (!pdbIdAlphaNum(pid, 3))
		return(false);

	// all tests passed		
	return(true);
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function pdbIdAlphaNum(pid, pos)
{
	var x = pid.charAt(pos);
	if ((x >= '0' && x <= '9') ||
		(x >= 'A' && x <= 'Z'))
		return true;
	else return false;
}
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
